'use strict';

const LambdaService = require('./lambda-service');

class DeployService {
  constructor(lambda = new LambdaService()) {
    this.lambda = lambda;

    this.getAliases = this.getAliases.bind(this);
    this.getVersions = this.getVersions.bind(this);
    this.updateAlias = this.updateAlias.bind(this);
  }

  async getAliases(fnName) {
    try {
      const { Aliases: aliases = [] } = await this.lambda.listAliases(fnName);

      return aliases.reduce((prev, curr) => {
        return {
          ...prev,
          [curr.Name]: curr.FunctionVersion,
        };
      }, {});
    } catch (err) {
      console.log('getAliases', err);
      throw err;
    }
  }

  async getVersions(fnName, marker) {
    try {
      const params = {
        FunctionName: fnName,
        MaxItems: 50,
      };

      if (marker) {
        params.Marker = marker;
      }

      const { NextMarker, Versions } = await this.lambda.listVersionsByFunction(params);

      if (NextMarker) {
        const versions = await this.getVersions(fnName, NextMarker);

        return Versions.concat(versions);
      }

      return Versions;
    } catch (err) {
      console.log('getVersions', err);
      throw err;
    }
  }

  async updateAlias(fnName, from, to) {
    try {
      if (!to) {
        throw new Error(`Alias 'to' is required, you've passed ${to}: ${typeof to} `);
      }
      from = from.toUpperCase();
      to = to.toUpperCase();

      let aliases = await this.getAliases(fnName);
      const aliasList = Object.keys(aliases);
      const versions = await this.getVersions(fnName).then(items => {
        return items.map(item => {
          return item.Version;
        });
      });

      const { ON_DECK, PROD, PREV_PROD } = DeployService.getLastVersions(versions);

      const latestVersion = ON_DECK;

      if (!aliasList.includes('PREV_PROD') && !aliasList.includes('PROD') && !aliasList.includes('ON_DECK')) {
        await this.lambda.createAlias(fnName, PREV_PROD, 'PREV_PROD');
        await this.lambda.createAlias(fnName, PROD, 'PROD');
        await this.lambda.createAlias(fnName, ON_DECK, 'ON_DECK');
        aliases = { ...aliases, PREV_PROD, PROD, ON_DECK };
        aliasList.push('PREV_PROD', 'PROD', 'ON_DECK');
        console.log(`Created default aliases for ${fnName}`);
      }

      if (!aliasList.includes(from) && from !== '$LATEST') {
        throw Error(`The from alias ${from} doesn't exist for ${fnName}`);
      }

      if (!aliasList.includes(to)) {
        await this.lambda.createAlias(fnName, latestVersion, to);
        aliasList.push(to);
        aliases = { ...aliases, [to]: latestVersion };
        console.log(`Created destiny alias ${to} for function ${fnName}`);
      }

      await this.lambda.updateAlias(fnName, from === '$LATEST' ? latestVersion : aliases[from], to);

      return DeployService.createUpdateObject(fnName, from, to, aliases, latestVersion);
    } catch (err) {
      console.log('updateAlias', err);
      throw err;
    }
  }

  static getLastVersions(versions) {
    if (versions.length < 1) {
      throw new Error(`Specified lambda doesn't have any versions yet`);
    }
    const length = versions.length;

    const last = versions[length - 1];
    const beforeLast = versions[length - 2] || last;
    const beforeBeforeLast = versions[length - 3] || beforeLast;

    return { ON_DECK: last, PROD: beforeLast, PREV_PROD: beforeBeforeLast };
  }

  static createUpdateObject(fnName, from, to, aliases, latestVersion) {
    return {
      name: fnName,
      from,
      to,
      fromVersion: from === '$LATEST' ? latestVersion : aliases[from],
      toVersion: aliases[to],
    };
  }
}

module.exports = DeployService;
